@@ -32,16 +32,16 @@
 #include "libdpx/DPXColorConverter.h"
 #include <OpenEXR/ImfTimeCode.h> //For TimeCode support
 
-#include "typedesc.h"
-#include "imageio.h"
-#include "fmath.h"
-#include "strutil.h"
+#include <OpenImageIO/typedesc.h>
+#include <OpenImageIO/imageio.h>
+#include <OpenImageIO/fmath.h>
+#include <OpenImageIO/strutil.h>
 #include <iomanip>
 
 OIIO_PLUGIN_NAMESPACE_BEGIN
 
 
-class DPXInput : public ImageInput {
+class DPXInput final : public ImageInput {
 public:
     DPXInput () : m_stream(NULL), m_dataPtr(NULL) { init(); }
     virtual ~DPXInput () { close(); }
@@ -100,6 +100,8 @@ OIIO_EXPORT ImageInput *dpx_input_imageio_create () { return new DPXInput; }
 
 OIIO_EXPORT int dpx_imageio_version = OIIO_PLUGIN_VERSION;
 
+OIIO_EXPORT const char* dpx_imageio_library_version () { return NULL; }
+
 OIIO_EXPORT const char * dpx_input_extensions[] = {
     "dpx", NULL
 };
@@ -191,8 +193,12 @@ DPXInput::seek_subimage (int subimage, int miplevel, ImageSpec &newspec)
     m_spec = ImageSpec (m_dpx.header.Width(), m_dpx.header.Height(),
         m_dpx.header.ImageElementComponentCount(subimage), typedesc);
 
-    m_spec.x = m_dpx.header.xOffset;
-    m_spec.y = m_dpx.header.yOffset;
+    // xOffset/yOffset are defined as unsigned 32-bit integers, but m_spec.x/y are signed
+    // avoid casts that would result in negative values
+    if (m_dpx.header.xOffset <= (unsigned int)std::numeric_limits<int>::max())
+        m_spec.x = m_dpx.header.xOffset;
+    if (m_dpx.header.yOffset <= (unsigned int)std::numeric_limits<int>::max())
+        m_spec.y = m_dpx.header.yOffset;
     if ((int)m_dpx.header.xOriginalSize > 0)
         m_spec.full_width = m_dpx.header.xOriginalSize;
     if ((int)m_dpx.header.yOriginalSize > 0)
@@ -204,25 +210,25 @@ DPXInput::seek_subimage (int subimage, int miplevel, ImageSpec &newspec)
         /*case dpx::kUserDefinedDescriptor:
             break;*/
         case dpx::kRed:
-            m_spec.channelnames.push_back("R");
+            m_spec.channelnames.emplace_back("R");
             break;
         case dpx::kGreen:
-            m_spec.channelnames.push_back("G");
+            m_spec.channelnames.emplace_back("G");
             break;
         case dpx::kBlue:
-            m_spec.channelnames.push_back("B");
+            m_spec.channelnames.emplace_back("B");
             break;
         case dpx::kAlpha:
-            m_spec.channelnames.push_back("A");
+            m_spec.channelnames.emplace_back("A");
             m_spec.alpha_channel = 0;
             break;
         case dpx::kLuma:
             // FIXME: do we treat this as intensity or do we use Y' as per
             // convention to differentiate it from linear luminance?
-            m_spec.channelnames.push_back("Y'");
+            m_spec.channelnames.emplace_back("Y'");
             break;
         case dpx::kDepth:
-            m_spec.channelnames.push_back("Z");
+            m_spec.channelnames.emplace_back("Z");
             m_spec.z_channel = 0;
             break;
         /*case dpx::kCompositeVideo:
@@ -234,8 +240,8 @@ DPXInput::seek_subimage (int subimage, int miplevel, ImageSpec &newspec)
             break;
         case dpx::kCbYCrY:
             if (m_wantRaw) {
-                m_spec.channelnames.push_back("CbCr");
-                m_spec.channelnames.push_back("Y");
+                m_spec.channelnames.emplace_back("CbCr");
+                m_spec.channelnames.emplace_back("Y");
             } else {
                 m_spec.nchannels = 3;
                 m_spec.default_channel_names ();
@@ -243,9 +249,9 @@ DPXInput::seek_subimage (int subimage, int miplevel, ImageSpec &newspec)
             break;
         case dpx::kCbYACrYA:
             if (m_wantRaw) {
-                m_spec.channelnames.push_back("CbCr");
-                m_spec.channelnames.push_back("Y");
-                m_spec.channelnames.push_back("A");
+                m_spec.channelnames.emplace_back("CbCr");
+                m_spec.channelnames.emplace_back("Y");
+                m_spec.channelnames.emplace_back("A");
                 m_spec.alpha_channel = 2;
             } else {
                 m_spec.nchannels = 4;
@@ -254,18 +260,18 @@ DPXInput::seek_subimage (int subimage, int miplevel, ImageSpec &newspec)
             break;
         case dpx::kCbYCr:
             if (m_wantRaw) {
-                m_spec.channelnames.push_back("Cb");
-                m_spec.channelnames.push_back("Y");
-                m_spec.channelnames.push_back("Cr");
+                m_spec.channelnames.emplace_back("Cb");
+                m_spec.channelnames.emplace_back("Y");
+                m_spec.channelnames.emplace_back("Cr");
             } else
                 m_spec.default_channel_names ();
             break;
         case dpx::kCbYCrA:
             if (m_wantRaw) {
-                m_spec.channelnames.push_back("Cb");
-                m_spec.channelnames.push_back("Y");
-                m_spec.channelnames.push_back("Cr");
-                m_spec.channelnames.push_back("A");
+                m_spec.channelnames.emplace_back("Cb");
+                m_spec.channelnames.emplace_back("Y");
+                m_spec.channelnames.emplace_back("Cr");
+                m_spec.channelnames.emplace_back("A");
                 m_spec.alpha_channel = 3;
             } else {
                 m_spec.default_channel_names ();
@@ -328,8 +334,10 @@ DPXInput::seek_subimage (int subimage, int miplevel, ImageSpec &newspec)
             break;
         case dpx::kUserDefined:
             if (! isnan (m_dpx.header.Gamma ()) && m_dpx.header.Gamma () != 0) {
-                m_spec.attribute ("oiio:ColorSpace", "GammaCorrected");
-                m_spec.attribute ("oiio:Gamma", (float) m_dpx.header.Gamma ());
+                float g = float(m_dpx.header.Gamma());
+                m_spec.attribute ("oiio:ColorSpace",
+                                  Strutil::format("GammaCorrected%.2g", g));
+                m_spec.attribute ("oiio:Gamma", g);
                 break;
             }
             // intentional fall-through
@@ -470,13 +478,13 @@ DPXInput::seek_subimage (int subimage, int miplevel, ImageSpec &newspec)
 
         int kc[7];
         get_keycode_values (kc);
-        m_spec.attribute("smpte:KeyCode", TypeDesc::TypeKeyCode, kc);
+        m_spec.attribute("smpte:KeyCode", TypeKeyCode, kc);
     }
 
     if (m_dpx.header.timeCode != 0xFFFFFFFF) {
 
         unsigned int timecode[2] = {m_dpx.header.timeCode, m_dpx.header.userBits};
-        m_spec.attribute("smpte:TimeCode", TypeDesc::TypeTimeCode, timecode);
+        m_spec.attribute("smpte:TimeCode", TypeTimeCode, timecode);
 
         // This attribute is dpx specific and is left in for backwards compatability.
         // Users should utilise the new smpte:TimeCode attribute instead
