@@ -33,30 +33,19 @@
 
 #include <iostream>
 
-// This needs to be included before GL.h
-// (which is included by QtOpenGL and QGLFormat)
-#include <glew.h>
-
 #include <OpenEXR/half.h>
 #include <OpenEXR/ImathFun.h>
 
-#include <QtGui/QComboBox>
-#include <QtGui/QLabel>
-#include <QtGui/QMouseEvent>
-#include <QtGui/QProgressBar>
-#include <QtOpenGL/QGLFormat>
-
-#include <boost/algorithm/string.hpp>
-using boost::algorithm::iequals;
-#include <boost/foreach.hpp>
-#include <boost/algorithm/string/split.hpp>
-#include <boost/algorithm/string/classification.hpp>
-#include <boost/algorithm/string/compare.hpp>
+#include <QComboBox>
+#include <QLabel>
+#include <QMouseEvent>
+#include <QProgressBar>
+#include <QGLFormat>
 
 #include "ivutils.h"
-#include "OpenImageIO/strutil.h"
-#include "OpenImageIO/fmath.h"
-#include "OpenImageIO/timer.h"
+#include <OpenImageIO/strutil.h>
+#include <OpenImageIO/fmath.h>
+#include <OpenImageIO/timer.h>
 
 
 static const char *
@@ -134,6 +123,13 @@ IvGL::initializeGL ()
     // Make sure initial matrix is identity (returning to this stack level loads
     // back this matrix).
     glLoadIdentity();
+
+#if 1
+    // Compensate for high res displays with device pixel ratio scaling
+    float dpr = m_viewer.devicePixelRatio();
+    glScalef (dpr, dpr, 1.0f);
+#endif
+
     // There's this small detail in the OpenGL 2.1 (probably earlier versions
     // too) spec:
     //
@@ -171,9 +167,9 @@ IvGL::create_textures (void)
     glGenTextures (total_texbufs, textures);
 
     // Initialize texture objects
-    for (int i = 0; i < total_texbufs; i++) {
-        m_texbufs.push_back (TexBuffer());
-        glBindTexture (GL_TEXTURE_2D, textures[i]);
+    for (unsigned int texture : textures) {
+        m_texbufs.emplace_back();
+        glBindTexture (GL_TEXTURE_2D, texture);
         GLERRPRINT ("bind tex");
         glTexImage2D (GL_TEXTURE_2D, 0 /*mip level*/,
                       4 /*internal format - color components */,
@@ -188,7 +184,7 @@ IvGL::create_textures (void)
         glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
         glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
         GLERRPRINT ("After tex parameters");
-        m_texbufs.back().tex_object = textures[i];
+        m_texbufs.back().tex_object = texture;
         m_texbufs.back().x = 0;
         m_texbufs.back().y = 0;
         m_texbufs.back().width = 0;
@@ -827,14 +823,15 @@ IvGL::paint_pixelview ()
 
         void *zoombuffer = alloca ((xend-xbegin)*(yend-ybegin)*nchannels*spec.channel_bytes());
         if (! m_use_shaders) {
-            img->get_pixels (spec.x + xbegin, spec.x + xend,
-                             spec.y + ybegin, spec.y + yend,
+            img->get_pixels (ROI (spec.x + xbegin, spec.x + xend,
+                                  spec.y + ybegin, spec.y + yend),
                              spec.format, zoombuffer);
         } else {
-            img->get_pixel_channels (spec.x + xbegin, spec.x + xend,
-                    spec.y + ybegin, spec.y + yend, 0, 1,
-                    m_viewer.current_channel(), m_viewer.current_channel()+nchannels,
-                    spec.format, zoombuffer);
+            ROI roi (spec.x + xbegin, spec.x + xend,
+                     spec.y + ybegin, spec.y + yend,
+                     0, 1,
+                     m_viewer.current_channel(), m_viewer.current_channel()+nchannels);
+            img->get_pixels (roi, spec.format, zoombuffer);
         }
 
         GLenum glformat, gltype, glinternalformat;
@@ -942,7 +939,7 @@ IvGL::useshader (int tex_width, int tex_height, bool pixelview)
 
     if (!m_use_shaders) {
         glTexEnvf (GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
-        BOOST_FOREACH (TexBuffer &tb, m_texbufs) {
+        for (auto&& tb : m_texbufs) {
             glBindTexture (GL_TEXTURE_2D, tb.tex_object);
             if (m_viewer.linearInterpolation ()) {
                 glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
@@ -1041,7 +1038,7 @@ IvGL::update ()
         glBindBufferARB (GL_PIXEL_UNPACK_BUFFER_ARB, 0);
     }
 
-    BOOST_FOREACH (TexBuffer &tb, m_texbufs) {
+    for (auto&& tb : m_texbufs) {
         tb.width = 0;
         tb.height= 0;
         glBindTexture (GL_TEXTURE_2D, tb.tex_object);
@@ -1481,7 +1478,7 @@ IvGL::typespec_to_opengl (const ImageSpec &spec, int nchannels, GLenum &gltype,
         break;
     }
 
-    bool issrgb = iequals (spec.get_string_attribute ("oiio:ColorSpace"), "sRGB");
+    bool issrgb = Strutil::iequals (spec.get_string_attribute ("oiio:ColorSpace"), "sRGB");
     
     glinternalformat = nchannels;
     if (nchannels == 1) {
@@ -1565,7 +1562,7 @@ IvGL::load_texture (int x, int y, int width, int height, float percent)
 {
     const ImageSpec &spec = m_current_image->spec ();
     // Find if this has already been loaded.
-    BOOST_FOREACH (TexBuffer &tb, m_texbufs) {
+    for (auto&& tb : m_texbufs) {
         if (tb.x == x && tb.y == y && tb.width >= width && tb.height >= height) {
             glBindTexture (GL_TEXTURE_2D, tb.tex_object);
             return;
@@ -1598,13 +1595,13 @@ IvGL::load_texture (int x, int y, int width, int height, float percent)
     // it safely since ImageBuf has a cache underneath and the whole image
     // may not be resident at once.
     if (! m_use_shaders) {
-        m_current_image->get_pixels (x, x + width, y, y + height,
+        m_current_image->get_pixels (ROI (x, x + width, y, y + height),
                                      spec.format, &m_tex_buffer[0]);
     } else {
-        m_current_image->get_pixel_channels (x, x+width, y, y+height, 0, 1,
-                                             m_viewer.current_channel(),
-                                             m_viewer.current_channel() + nchannels,
-                                             spec.format, &m_tex_buffer[0]);
+        m_current_image->get_pixels (ROI (x, x+width, y, y+height, 0, 1,
+                                          m_viewer.current_channel(),
+                                          m_viewer.current_channel() + nchannels),
+                                     spec.format, &m_tex_buffer[0]);
     }
     if (m_use_pbo) {
         glBindBufferARB (GL_PIXEL_UNPACK_BUFFER_ARB, 
